@@ -4,12 +4,12 @@ import contextlib
 from contextlib import AsyncExitStack, ExitStack
 from typing import (
     Any,
+    Awaitable,
     Callable,
     Dict,
     Iterable,
     List,
     Mapping,
-    NamedTuple,
     Tuple,
     TypeVar,
     Union,
@@ -28,11 +28,25 @@ from di.api.scopes import Scope
 from di.exceptions import IncompatibleDependencyError
 
 
-class ExecutionState(NamedTuple):
-    stacks: Mapping[Scope, Union[AsyncExitStack, ExitStack]]
-    results: List[Any]
-    cache: ScopeMap[CacheKey, Any]
-    values: Mapping[DependencyProvider, Any]
+class ExecutionState:
+    __slots__ = (
+        "stacks",
+        "results",
+        "values",
+        "cache",
+    )
+
+    def __init__(
+        self,
+        stacks: Mapping[Scope, Union[AsyncExitStack, ExitStack]],
+        results: List[Any],
+        cache: ScopeMap[CacheKey, Any],
+        values: Mapping[DependencyProvider, Any],
+    ):
+        self.stacks = stacks
+        self.results = results
+        self.cache = cache
+        self.values = values
 
 
 DependencyType = TypeVar("DependencyType")
@@ -53,7 +67,9 @@ class Task:
         "compute",
     )
 
-    compute: Any
+    compute: Union[
+        Callable[[ExecutionState], None], Callable[[ExecutionState], Awaitable[None]]
+    ]
     wrapped_call: DependencyProvider
     user_function: DependencyProvider
 
@@ -99,7 +115,7 @@ class Task:
                 self.compute = self.compute_sync_func_no_cache
 
         self.call_user_func_with_deps = self.generate_execute_fn(
-            self.wrapped_call, positional_parameters, keyword_parameters
+            positional_parameters, keyword_parameters
         )
 
     def __hash__(self) -> int:
@@ -107,10 +123,9 @@ class Task:
 
     def generate_execute_fn(
         self,
-        call: DependencyProvider,
         positional_parameters: Iterable[Task],
         keyword_parameters: Iterable[Tuple[str, Task]],
-    ) -> Callable[[List[Any]], Any]:
+    ) -> Callable[[Callable[..., Any], List[Any]], Any]:
         # this codegen speeds up argument collection and passing
         # by avoiding creation of intermediary containers to store the values
         positional_arg_template = "results[{}]"
@@ -120,10 +135,9 @@ class Task:
             args.append(positional_arg_template.format(task.task_id))
         for keyword, task in keyword_parameters:
             args.append(keyword_arg_template.format(keyword, task.task_id))
-        lcls: Dict[str, Any] = {}
-        glbls = {"call": call}
-        exec(f'def execute(results): return call({",".join(args)})', glbls, lcls)
-        return lcls["execute"]  # type: ignore[no-any-return]
+        out: Dict[str, Callable[..., Any]] = {}
+        exec(f'def execute(call, results): return call({",".join(args)})', out)
+        return out["execute"]
 
     def __repr__(self) -> str:
         return (
@@ -138,7 +152,9 @@ class Task:
         if value is not UNSET:
             state.results[self.task_id] = value
             return
-        dependency_value = await self.call_user_func_with_deps(state.results)
+        dependency_value = await self.call_user_func_with_deps(
+            self.wrapped_call, state.results
+        )
         state.results[self.task_id] = dependency_value
         state.cache.set(self.cache_key, dependency_value, scope=self.scope)
 
@@ -146,7 +162,9 @@ class Task:
         if self.user_function in state.values:
             state.results[self.task_id] = state.values[self.user_function]
             return
-        dependency_value = await self.call_user_func_with_deps(state.results)
+        dependency_value = await self.call_user_func_with_deps(
+            self.wrapped_call, state.results
+        )
         state.results[self.task_id] = dependency_value
 
     async def compute_async_cm_cache(self, state: ExecutionState) -> None:
@@ -166,7 +184,7 @@ class Task:
             ) from None
 
         dependency_value: Any = await enter(
-            self.call_user_func_with_deps(state.results)
+            self.call_user_func_with_deps(self.wrapped_call, state.results)
         )
         state.results[self.task_id] = dependency_value
         state.cache.set(self.cache_key, dependency_value, scope=self.scope)
@@ -183,7 +201,7 @@ class Task:
                 f" and canot be used in the sync scope {self.scope}"
             ) from None
         dependency_value: Any = await enter(
-            self.call_user_func_with_deps(state.results)
+            self.call_user_func_with_deps(self.wrapped_call, state.results)
         )
         state.results[self.task_id] = dependency_value
 
@@ -196,7 +214,7 @@ class Task:
             state.results[self.task_id] = value
             return
         val = state.stacks[self.scope].enter_context(
-            self.call_user_func_with_deps(state.results)
+            self.call_user_func_with_deps(self.wrapped_call, state.results)
         )
         state.results[self.task_id] = val
         state.cache.set(self.cache_key, val, scope=self.scope)
@@ -206,7 +224,7 @@ class Task:
             state.results[self.task_id] = state.values[self.user_function]
             return
         state.results[self.task_id] = state.stacks[self.scope].enter_context(
-            self.call_user_func_with_deps(state.results)
+            self.call_user_func_with_deps(self.wrapped_call, state.results)
         )
 
     def compute_sync_func_cache(self, state: ExecutionState) -> None:
@@ -217,7 +235,7 @@ class Task:
         if value is not UNSET:
             state.results[self.task_id] = value
             return
-        val = self.call_user_func_with_deps(state.results)
+        val = self.call_user_func_with_deps(self.wrapped_call, state.results)
         state.results[self.task_id] = val
         state.cache.set(self.cache_key, val, scope=self.scope)
 
@@ -226,4 +244,6 @@ class Task:
             state.results[self.task_id] = state.values[self.user_function]
             return
 
-        state.results[self.task_id] = self.call_user_func_with_deps(state.results)
+        state.results[self.task_id] = self.call_user_func_with_deps(
+            self.wrapped_call, state.results
+        )
