@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 from contextlib import contextmanager
 from typing import (
     Any,
@@ -25,12 +27,12 @@ DependencyType = TypeVar("DependencyType")
 
 
 class Container:
-    __slots__ = ("_bind_hooks", "_state")
+    __slots__ = ("_register_hooks", "_state")
 
-    _bind_hooks: List[BindHook]
+    _register_hooks: List[BindHook]
 
     def __init__(self) -> None:
-        self._bind_hooks = []
+        self._register_hooks = []
 
     def bind(
         self,
@@ -45,14 +47,14 @@ class Container:
         the scope or any other data from the dependency they are replacing.
         """
 
-        self._bind_hooks.append(hook)
+        self._register_hooks.append(hook)
 
         @contextmanager
-        def unbind() -> "Generator[None, None, None]":
+        def unbind() -> Generator[None, None, None]:
             try:
                 yield
             finally:
-                self._bind_hooks.remove(hook)
+                self._register_hooks.remove(hook)
 
         return unbind()
 
@@ -61,7 +63,7 @@ class Container:
         dependency: DependantBase[DependencyType],
         scopes: Sequence[Scope],
     ) -> SolvedDependant[DependencyType]:
-        return solve(dependency, scopes, self._bind_hooks)
+        return solve(dependency, scopes, self._register_hooks)
 
     def execute_sync(
         self,
